#######################################
#Author      : Annette Janssen
#Institution : Wageningen University
#Date        : May 2021
#######################################

rm(list = ls())
#.libPaths("PATHLIBRARY")

library(ncdf4)
library(abind)
library(readxl)
library(ggplot2)
library(colorspace)

mod_dir           <- "./VICAnalysis/"
val_dir           <- "./Validation/MODISdata/" #(https://doi.org/10.5067/MODIS/MOD11A1.006), downloaded and preprocessed in excel
val_dir_lakes     <- "./Validation/MODISdata/Taiihu" #data from sun et al 2010: Sun, H., Qin, B. and Hu, C. (2010) Ecosystem dataset of lakes and wetlands: Jiangsu Taihu Lake Station (1991-2006) (Chinese: 中国生态系统定位观测与研究数据集 湖泊湿地海湾生态系统卷: 江苏太湖站（1991-2006)), China Agriculture Press.
inp_dir           <- "./InputData"
VICOUTfolder      <- "./VICoutput/"
FIGURESFOLDER     <- "./Figures"
ANALYSISFOLDER    <- "./AnalysisData"
OUTPUTFILES       <- list.files(mod_dir)
OUTPUTFILES       <- OUTPUTFILES[regexpr('.csv', OUTPUTFILES)==-1]
MODELS            <- OUTPUTFILES[regexpr('Paper',OUTPUTFILES)==-1][3:6]
YEARS             <- c(2001:2005)
MONTHSinYEAR      <- unique(months((as.Date(c(0:364), origin = "2001-01-01"))))
MONTHSbyDAY       <- months((as.Date(c(0:364), origin = "2001-01-01")))
valKEY            <- read_excel(paste(val_dir,"Validation_Data.xlsx",sep = ""),sheet = "KeySheet",col_types = "numeric")
valDATA           <- read_excel(paste(val_dir,"Validation_Data.xlsx",sep = ""),sheet = "TemperatureSheet",col_types = "numeric")
DATA              <- readRDS(paste(ANALYSISFOLDER,"/DATAtable2.csv",sep=""))
IDlarge           <- c(151,148,1430,1483,1417)

valDATA_taihu     <- read_excel(paste(val_dir_lakes,"THL_waterdata.xlsx",sep = ""),sheet = "Temperature",col_types = "numeric")
HLid_Taihu        <- 148

INPUTarray        <- readRDS(paste(inp_dir,"/INPUTarray.csv",sep=""))
ID                <- c(1:length(valKEY$Hylak_id))*0

for (i in c(1:length(valKEY$Hylak_id))){
  ID[i]<- which(INPUTarray$HLid==valKEY$Hylak_id[i])
}
ID<-INPUTarray$VICid[ID]+1


aMODEL      <- MODELS[1]
aYEAR       <- YEARS[1]
clnames     <- c("TongID","HydlID","key","Model","Year","Month","Temperature")
df          <- array(0,c(0,length(clnames)))
colnames(df)<- clnames

for (aMODEL in MODELS){
  print(aMODEL)
  aMODEL_dir <- paste(mod_dir,aMODEL,sep = "")
  for (aYEAR in YEARS){
    chk <- 0
    print(aYEAR)
    filename   <- paste(aMODEL,"_",aYEAR,"_TSurf.csv",sep = "")
    DATAYEAR   <- read.csv(paste(aMODEL_dir,"/",filename,sep = ""),header=TRUE)
    DATAYEAR   <- DATAYEAR[,2:366]
    DATAYEAR_ID<- DATAYEAR[ID,]
    
    
    filenames_hi            <- list.files(VICOUTfolder,paste(aMODEL,"_historical.1861-01-01.nc",sep=""))
    nc.OUTPUT               <- nc_open(paste(VICOUTfolder,filenames_hi,sep = ""), write=FALSE, readunlim=TRUE, verbose=FALSE,
                                       auto_GMT=TRUE, suppress_dimvals=FALSE )
    if (chk==0){
      OUTPUT.dates            <- as.Date(nc.OUTPUT$dim$time$vals, origin = "0000-12-30")
      OUTPUT.months           <- as.numeric(format.Date(OUTPUT.dates, "%m"))
      OUTPUT.years            <- as.numeric(format.Date(OUTPUT.dates, "%Y"))
      chk <- 1
    }
    
    IceTemp<-ncvar_get(nc = nc.OUTPUT,varid = "OUT_LAKE_ICE_TEMP",start=c(1,min(which(OUTPUT.years==aYEAR))),count=c(-1,365))
    IceTemp_ID<- IceTemp[ID,]
    
    #DATAMONTH <- array(0,c(length(valKEY$Hylak_id),12))
    for (mth in c(1:12)){
      DATAMONTH <- rowMeans(DATAYEAR_ID[,which(MONTHSbyDAY==MONTHSinYEAR[mth])])
      
      IceTempMTH<- rowMeans(IceTemp_ID[,which(MONTHSbyDAY==MONTHSinYEAR[mth])])
      DATAMONTH[IceTempMTH<0]<-0
      
      df_dum           <- data.frame(TongID<- valKEY$Lake_Number_Tong,
                                     HydID <- valKEY$Hylak_id,
                                     key   <- paste(valKEY$Lake_Number_Tong,"_",aYEAR,"_",mth,sep = ""),
                                     model <- aMODEL,
                                     year  <- aYEAR,
                                     month <- mth,
                                     Temperature <-DATAMONTH)
      colnames(df_dum) <- clnames
      df               <-rbind(df,df_dum)
    }
  }
}


IDval<-valDATA[which(valDATA$Year<2006),]
IDval2 <- cbind(IDval,paste(IDval$Lake_Number_Tong,"_",IDval$Year,"_",IDval$Month,sep = ""))
colnames(IDval2)<-c(colnames(IDval),"key")

df_merged<-merge(df, IDval2, by = "key", all.x=TRUE, all.y=TRUE)

dfcor<-df_merged[which(df_merged$Temperature>0),]


SU_All<-summary(lm(formula = dfcor$Temperature~dfcor$Avarage_temp))
SU_1<-summary(lm(formula = dfcor$Temperature[dfcor$Model==MODELS[1]]~dfcor$Avarage_temp[dfcor$Model==MODELS[1]]))
SU_2<-summary(lm(formula = dfcor$Temperature[dfcor$Model==MODELS[2]]~dfcor$Avarage_temp[dfcor$Model==MODELS[2]]))
SU_3<-summary(lm(formula = dfcor$Temperature[dfcor$Model==MODELS[3]]~dfcor$Avarage_temp[dfcor$Model==MODELS[3]]))
SU_4<-summary(lm(formula = dfcor$Temperature[dfcor$Model==MODELS[4]]~dfcor$Avarage_temp[dfcor$Model==MODELS[4]]))

#cmaps<-c('black','gray50'
# ,'steelblue1','steelblue3',
# 'olivedrab1','olivedrab3',
# 'sienna1','sienna3',
# 'gold1','gold3')
cmaps<-c('black'
         ,'steelblue3','skyblue',
         'olivedrab3','olivedrab1',
         'sienna3','sienna1',
         'gold3','gold1','gray20')

fig_validation<- 
  ggplot()+
  geom_point(data=dfcor,aes(y=Temperature,x=Avarage_temp,color=Model),alpha=1)+
  geom_smooth(data=dfcor,aes(y=Temperature,x=Avarage_temp,color=paste(Model,2)),method= "lm",formula =y~x,size=1.3)+
  geom_smooth(data=dfcor,aes(y=Temperature,x=Avarage_temp,color="Model average"),method= "lm",formula =y~x,size=1.3)+
  geom_line(aes(x=c(0:31),y=c(0:31),color="1:1 Line"),size=1,linetype = "dashed")+
  scale_color_manual(name = "Legend",values=cmaps) +
  xlab("Temperature predicted VIC-LAKE (?C)")+
  ylab("Temperature observed MODIS (?C)")+
  theme(panel.background = element_rect(fill = "white",
                                        colour = "white",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        axis.text=element_text(size=20),
        axis.title=element_text(size=25),
        legend.position = "right")+  
  geom_text(aes(x=1,y=30, label=paste(MODELS[1]),hjust = 0),size=6)+
  geom_text(aes(x=10,y=30,label=paste(": y= ",round(SU_1$coefficients[1],2),"+",round(SU_1$coefficients[2],2),
                                      "x, R2_adj=",round(SU_1$adj.r.squared,2),"***",sep=""),hjust = 0),size=6)+
  geom_text(aes(x=1,y=29, label=paste(MODELS[2]),hjust = 0),size=6)+
  geom_text(aes(x=10,y=29,label=paste(": y=",round(SU_2$coefficients[1],2),"+",round(SU_2$coefficients[2],2),
                                      "x, R2_adj=",round(SU_2$adj.r.squared,2),"***",sep=""),hjust = 0),size=6)+
  geom_text(aes(x=1,y=28, label=paste(MODELS[3]),hjust = 0),size=6)+
  geom_text(aes(x=10,y=28,label=paste(": y=",round(SU_3$coefficients[1],2),"+",round(SU_3$coefficients[2],2),
                                      "x, R2_adj=",round(SU_3$adj.r.squared,2),"***",sep=""),hjust = 0),size=6)+
  geom_text(aes(x=1,y=27, label=paste(MODELS[4]),hjust = 0),size=6)+
  geom_text(aes(x=10,y=27,label=paste(": y=",round(SU_4$coefficients[1],2),"+",round(SU_4$coefficients[2],2),
                                      "x, R2_adj=",round(SU_4$adj.r.squared,2),"***",sep=""),hjust = 0),size=6)+
  geom_text(aes(x=1,y=26, label=paste("Average"),hjust = 0),size=6)+
  geom_text(aes(x=10,y=26,label=paste(": y=",round(SU_All$coefficients[1],2),"+",round(SU_All$coefficients[2],2),
                                      "x, R2_adj=",round(SU_All$adj.r.squared,2),"***",sep=""),hjust = 0),size=6)+
  xlim(c(0,35))+ylim(c(0,35))

pdf(paste(FIGURESFOLDER,"/FIGURE_validationTemp.pdf",sep=""),height=12,width=18)
print(fig_validation)
dev.off()  

############################
INPUTarray        <- readRDS(paste(inp_dir,"/INPUTarray.csv",sep=""))
ID2               <- c(1:length(IDlarge))*0
NM                <- rep("No Name", 4)
for (i in c(1:length(IDlarge))){
  ID2[i]<- which(INPUTarray$HLid==IDlarge[i])
  NM[i]<-as.character(DATA$LakeName[which(DATA$HLid==IDlarge[i])])
}
ID2<-INPUTarray$VICid[ID2]+1


aMODEL       <- MODELS[1]
aYEAR        <- YEARS[1]
df2          <- array(0,c(0,length(clnames)))
colnames(df2)<- clnames

for (aMODEL in MODELS){
  print(aMODEL)
  aMODEL_dir <- paste(mod_dir,aMODEL,sep = "")
  for (aYEAR in YEARS){
    chk <- 0
    print(aYEAR)
    filename   <- paste(aMODEL,"_",aYEAR,"_TSurf.csv",sep = "")
    DATAYEAR   <- read.csv(paste(aMODEL_dir,"/",filename,sep = ""),header=TRUE)
    DATAYEAR   <- DATAYEAR[,2:366]
    DATAYEAR_ID<- DATAYEAR[ID2,]
    
    df_dum2     <- data.frame (key   <- paste(aYEAR,"_",c(1:365),sep = ""),
                               model <- aMODEL,
                               year  <- aYEAR,
                               month <- c(1:365),
                               as.Date(c(1:365),origin = paste(aYEAR,"-","01-01",sep="")),
                               Temperature <-t(DATAYEAR_ID))
    match(MONTHSbyDAY, month.name)
    
    colnames(df_dum2) <- c("key"     ,    "Model"   ,    "Year",  "Day","time"     , NM)
    df2               <-rbind(df2,df_dum2)
  }
}

IDval<-valDATA[which(valDATA$Year<2006),]
for (i in c(1:length(IDlarge))){
  IDlake<-valKEY$Lake_Number_Tong[which(valKEY$Hylak_id==IDlarge[i])]
  IDlake<-which(IDval$Lake_Number_Tong==IDlake)
  df_LD<-data.frame(IDlarge[i],
                    IDval$Lake_Number_Tong[IDlake],
                    IDval$Year[IDlake],
                    IDval$Month[IDlake],
                    as.Date(paste(IDval$Year[IDlake],"-",IDval$Month[IDlake],"-15",sep = ""), "%Y-%m-%d"),
                    IDval$Avarage_temp[IDlake])
  colnames(df_LD)<- c("HLID"    ,    "LocalID"   ,    "Year","month"     ,   "time","temperature")
  
  Fig_lakes<- ggplot()+
    geom_line(aes(x=df2$time,y=df2[,5+i], color=df2$Model))+
    geom_point(aes(x=df_LD$time,y=df_LD$temperature),color='black')+
    xlab('Time (yr)')+
    ylab('Temperature (?C)')+
    theme(panel.background = element_rect(fill = "white",
                                          colour = "white",
                                          size = 0, linetype = "solid"),
          panel.grid.major = element_line(size = 0, 
                                          linetype = 'solid',
                                          colour = "white"))+
    ylim(0,35)+
    labs(title=DATA$LakeName[which(DATA$HLid==IDlarge[i])])
  
  pdf(paste(FIGURESFOLDER,"/FIGURE_HLLakeID_",IDlarge[i],"_validationTemp.pdf",sep=""),height=6,width=6)
  print(Fig_lakes)
  dev.off()  
}



################################
INPUTarray        <- readRDS(paste(inp_dir,"/INPUTarray.csv",sep=""))
ID3<- INPUTarray$VICid[which(INPUTarray$HLid==IDlarge[2])]+1
NM3<-as.character(DATA$LakeName[which(DATA$HLid==IDlarge[2])])

aMODEL       <- MODELS[1]
aYEAR        <- YEARS[1]
df3          <- array(0,c(0,length(clnames)))
colnames(df3)<- clnames

for (aMODEL in MODELS){
  print(aMODEL)
  aMODEL_dir <- paste(mod_dir,aMODEL,sep = "")
  for (aYEAR in 1991:2005){
    chk <- 0
    print(aYEAR)
    filename   <- paste(aMODEL,"_",aYEAR,"_TSurf.csv",sep = "")
    DATAYEAR   <- read.csv(paste(aMODEL_dir,"/",filename,sep = ""),header=TRUE)
    DATAYEAR   <- DATAYEAR[,2:366]
    DATAYEAR_ID<- DATAYEAR[ID3,]
    
    df_dum3     <- data.frame (key   <- paste(aYEAR,"_",c(1:365),sep = ""),
                               model <- aMODEL,
                               year  <- aYEAR,
                               month <- c(1:365),
                               as.Date(c(1:365),origin = paste(aYEAR,"-","01-01",sep="")),
                               Temperature <-t(DATAYEAR_ID))
    match(MONTHSbyDAY, month.name)
    
    colnames(df_dum3) <- c("key"     ,    "Model"   ,    "Year",  "Day","time"     , NM3)
    df3               <-rbind(df3,df_dum3)
  }
}

colnames(df3)<- c("key"     ,    "Model"   ,    "Year",  "Day","time"     , NM3)
IDval_taihu<-valDATA_taihu[which(valDATA_taihu$year<2006),]
IDval2_taihu <- cbind(IDval_taihu,paste(HLid_Taihu,"_",IDval_taihu$year,"_",IDval_taihu$month,sep = ""),
                      as.Date(paste(IDval_taihu$year,"-",IDval_taihu$month,"-01",sep = ""), "%Y-%m-%d"))
colnames(IDval2_taihu)<-c(colnames(valDATA_taihu),"key","day_num")
IDval3_taihu<-na.omit(IDval2_taihu[,c(1:4,6:13)])


fig_Taihu<-ggplot()+
  geom_line(aes(x=df3$time,y=df3$Tai, color=df3$Model))+
  geom_point(aes(x=IDval3_taihu$day_num,
                 y=IDval3_taihu[,10]),color= 'black')+
  xlab('Time (yr)')+
  ylab('Temperature (?C)')+
  theme(panel.background = element_rect(fill = "white",
                                        colour = "white",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"))+
  ylim(0,35)

pdf(paste(FIGURESFOLDER,"/FIGURE_Taihu_validationTemp.pdf",sep=""),height=6,width=6)
print(fig_Taihu)
dev.off()  

